#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <ctype.h>

#include <stdarg.h>

#include <vector>
#include <string>

#include <amsstring/amsstring.hpp>

using namespace ams;
using namespace std;

namespace ams
{

void amsstring_tests1()
{
  amsstring q;
  amsstring q2("this is a test.\n");
  amsstring q3(10,'a');

  printf("length of q = %d\n",q.length);
  printf("q2: %s\n",q2.cstring);
  printf("q3: %s\n",q3.cstring);

  q2.resize(5);
  q3.resize(5);
  printf("q2: %s\n",q2.cstring);
  printf("q3: %s\n",q3.cstring);

  q2.append("some more stuff");
  q3.append(&q2);
  printf("q2: %s\n",q2.cstring);
  printf("q3: %s\n",q3.cstring);
  q2.copy(&q3);
  printf("q2: %s\n",q2.cstring);
  printf("q3: %s\n",q3.cstring);

  //memory leak!
  q.insert(5,":a string inserted at 5:");
  q3.insert(0,"anotherstring inserted at 0:");
  q2.insert(5,":anotherstring inserted at 5:");
  printf("q: %s\n",q.cstring);
  printf("q2: %s\n",q2.cstring);
  printf("q3: %s\n",q3.cstring);

  stripwhitespace(&q);
  printf("q: %s\n",q.cstring);
  stripallwhitespace(&q);
  printf("q: %s\n",q.cstring);
  q.remove(5);
  printf("q: %s\n",q.cstring);
  q.remove(5);
  printf("q: %s\n",q.cstring);
  q.remove(5);
  printf("q: %s\n",q.cstring);

  int ind;
  ind  = q2.find("inserted");
  printf("q2: 'inserted' found at %d:\n",ind);
  ind  = q2.find("inserted",ind+1);
  printf("q2: next 'inserted' found at %d:\n",ind);
  ind  = q2.find("inserted");
  q2.substring(ind,ind+strlen("inserted")-1,&q3);
  printf("q3: %s:\n",q3.cstring);
  q2.remove(ind,ind+strlen("inserted")-1);
  printf("q2: %s:\n",q2.cstring);

  q2.remove(q2.length);
  q2.remove(q2.length+1);
  printf("q2: %s:\n",q2.cstring);

  std::vector<ams::amsstring> lns;
  int I;

  splitwhitespace(&q2,&lns);
  printf("Components of q2:\n");
  for(I=0;I<lns.size();I++)
    printf("%d: %s\n",I,lns[I].cstring);

  q.copy("A csv string,1.6,1.2,55.55,1.0E-19,,2");
  printf("CSV test: %s\n",q.cstring);
  split(&q,',',&lns);
  printf("Components of q:\n");
  for(I=0;I<lns.size();I++)
    printf("%d: %s\n",I,lns[I].cstring);
  q.sprintf(50,"CSV test2: %1.6g,%1.6g,",55.55,55.56);
  printf("CSV test: %s\n",q.cstring);
  split(&q,',',&lns);
  printf("Components of q:\n");
  for(I=0;I<lns.size();I++)
    printf("%d: %s\n",I,lns[I].cstring);

  q.sprintf(100,"a wakka delimited wakka string wakka 2");
  printf("string delimited test: %s\n",q.cstring);
  split(&q,"wakka",&lns);
  printf("Components of q:\n");
  for(I=0;I<lns.size();I++)
    printf("%d: %s\n",I,lns[I].cstring);

  q.sprintf(100,"A multiline\nstring in the UNIX\n style...\n");
  printf("Multiline test\n%s\n",q.cstring); //memory leak
  splitlines(&q,&lns);
  for(I=0;I<lns.size();I++)
   printf("%d: %s\n",I,lns[I].cstring);
}

void amsstring_tests2()
{
  amsstring q;
  std::vector<amsstring> lns;
  int I;

  FILE *fp;
  FILE *fp2;

  fp = fopen("testtext1.txt","r+");

  for(I=0;I<10;I++)
  {
    freadline(fp,&q);
    printf("%d: %s\n",I,q.cstring);
  }

  fseek(fp,SEEK_SET,0);

  freadlines(fp,&lns);
  for(I=0;I<10;I++)
  {
    printf("%d: %s\n",I,lns[I].cstring);
  }
  printf("and %d lines total...\n",lns.size());

  fclose(fp);

  fp2 = fopen("testtxt2.txt","w+");
  for(I=0;I<lns.size();I++)
  {
    if(I%2==0)
    {
      q = lns[I];
      q.append("\n");
      //printf("%s",q.cstring);
      fwritelines(fp2,&q);
    }
  }

  fclose(fp2);

  fp = fopen("testtext1.txt","r+");
  if(fp==NULL)
  {
    printf("We have a problem...\n");
  }
  freadtxtfile(fp,&q);

  printf("%s",q.cstring);

  fclose(fp);

  return;
}

void amsstring_tests3()
{
  amsstring a = "this is a test";
  amsstring b = "this is a tes";

  printf("String assignment and boolean tests:\n");
  printf("a: %s\n",a.cstring);
  printf("b: %s\n",b.cstring);
  printf("a==b : %d\n",a==b);
  a = b;
  printf("a = b, a==b: %d\n",a==b);
  printf("find 'wakka' in a: %d\n",a.find("wakka"));
  printf("find 'is a' in a: %d\n",a.find("is a"));


  return;
}

void amsstring_tests4()
{
  int I;
  printf("amsstring: tests 4\n");
  amsstring a = "";
  amsstring b = "Hello world\n";
  printf("a: %s\n",a.cstring);
  printf("b: %s\n",b.cstring);
  for(I=0;I<b.size();I++)
  {
    a.append(b.cstring[I]);
    printf("b[I]: %c, a: %s\n",b.cstring[I],a.cstring);
  }
  return;
}

void amsstring_tests5()
{
  amsstring a,b,c,d;

  a = "Hello World!";
  b = "Bbb";
  c = "Bbc";
  d = "Bbc ";

  printf("a = %s\n",a.cstring);
  printf("b = %s\n",b.cstring);
  printf("c = %s\n",c.cstring);
  printf("d = %s\n",d.cstring);
  printf("a==b: %d, a!=c:%d\n",a==b,a!=c);
  printf("a<b:%d\n",a<b);
  printf("a>b:%d\n",a>b);
  printf("b<c:%d\n",b<c);
  printf("c<d:%d\n",c<d);

  b = "Hello World!";

  printf("a = %s\n",a.cstring);
  printf("b = %s\n",b.cstring);
  printf("a<b:%d\n",a<b);
  printf("a>b:%d\n",a>b);
  printf("a[1]=%c, a[2]%c\n",a[1],a[2]);

  return;
}











};
