#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <cstring>
#include <cstdio>
#include <stdarg.h>

//Portability layer
//The GNU compilers do not define strcpy_s and sprintf_s - these are microsoft only functions.
//Damn them to hell
//These two functions check if I'm compiling on windows with mingw-64, or on linux.

//need portable versions of strcpy_s and sprintf_s

namespace ams
{

//strcpy_s
//strncpy
int amsstrcpy_s(char *dest, int size, const char *src)
{
  int ret = 0;

  #if defined(LINUX) || defined(linux) || defined(__linux__) || defined(__GNUC__)
    //use strncpy
    strncpy(dest,src,size);
    ret = 0;
    dest[size-1] = '\0';

  #elif defined(__MINGW32__) || defined(__MINGW64__)
    //use strcpy_s
    ret = (int)strcpy_s(dest,size,src);

  #else
    #pragma message("amsstrcpy_s: Unsupported architecture - neither linux nor mingw64 nor msvc")
  #endif

  return ret;
}

//sprintf_s
//snprintf
//
int amssprintf_s(char *s, int n, const char *format, ...)
{
  int ret = 0;

  va_list args;
  va_start(args, format);
  //exampleV(b, args);
  //va_arg(val,datatype)

  #if defined(LINUX) || defined(linux) || defined(__linux__) || defined(__GNUC__)
    //use snprintf
    ret = (int)vsnprintf(s,n,format,args);
    s[n-1] = '\0';

  #elif defined(__MINGW32__) || defined(__MINGW64__) || defined(_WIN32)
    //use sprintf_s
    ret = (int)vsprintf_s(s,n,format,args);
  #else
    #pragma message("amssprintf_s: Unsupported architecture - neither linux nor mingw64 nor msvc")

  #endif


  va_end(args);
  return ret;
}


};
